"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _helpers2 = require("./helpers");

var _helpers3 = _interopRequireDefault(_helpers2);

exports["default"] = function (_ref) {
  var Plugin = _ref.Plugin;
  var t = _ref.types;

  var visitor = {};

  var _helpers = (0, _helpers3["default"])(t);

  var buildChildren = _helpers.buildChildren;
  var extractOpenArguments = _helpers.extractOpenArguments;
  var flattenExpressions = _helpers.flattenExpressions;
  var toReference = _helpers.toReference;
  var toFunctionCall = _helpers.toFunctionCall;
  var toFunctionCallStatement = _helpers.toFunctionCallStatement;
  var attrsToAttrCalls = _helpers.attrsToAttrCalls;

  visitor.JSXOpeningElement = {
    exit: function exit(_ref2, parent, scope) {
      var name = _ref2.name;
      var attributes = _ref2.attributes;
      var selfClosing = _ref2.selfClosing;

      var tag = toReference(name);
      var args = [tag];

      var _extractOpenArguments = extractOpenArguments(attributes);

      var key = _extractOpenArguments.key;
      var statics = _extractOpenArguments.statics;
      var attrs = _extractOpenArguments.attrs;
      var hasSpread = _extractOpenArguments.hasSpread;

      var elementFunction = selfClosing ? "elementVoid" : "elementOpen";

      // Only push arguments if they're needed
      if (key || statics || attrs) {
        args.push(key || t.literal(null));
      }
      if (statics || attrs) {
        args.push(statics ? t.arrayExpression(statics) : t.literal(null));
      }

      // If there is a spread element, we need to use
      // the elementOpenStart/elementOpenEnd syntax.
      // This allows spreads to be transformed into
      // attr(name, value) calls.
      if (hasSpread) {
        attrs = attrs.map(attrsToAttrCalls(scope));

        return t.sequenceExpression([toFunctionCall("elementOpenStart", args)].concat(_toConsumableArray(attrs), [toFunctionCall("elementOpenEnd", [tag])]));
      } else if (attrs) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = attrs[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var _step$value = _slicedToArray(_step.value, 2);

            var _name = _step$value[0];
            var value = _step$value[1];

            args.push(_name, value);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator["return"]) {
              _iterator["return"]();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }

      return toFunctionCallStatement(elementFunction, args);
    }
  };

  visitor.JSXClosingElement = {
    exit: function exit(_ref3) {
      var name = _ref3.name;

      return toFunctionCallStatement("elementClose", [toReference(name)]);
    }
  };

  visitor.JSXElement = {
    exit: function exit(_ref4) {
      var openingElement = _ref4.openingElement;
      var children = _ref4.children;
      var closingElement = _ref4.closingElement;

      // Filter out empty children, and transform JSX expressions
      // into normal expressions.
      children = buildChildren(children);

      var elements = [openingElement].concat(_toConsumableArray(children));
      if (closingElement) {
        elements.push(closingElement);
      }

      // Turn all sequence expressions into function statements.
      return flattenExpressions(elements);
    }
  };

  return new Plugin("incremental-dom", { visitor: visitor });
};

module.exports = exports["default"];